#!/usr/bin/perl5
# Count Acrobat 5 Annotations in T10 letter ballot comment resolution
# by Rob Elliott, HP
# elliott@hp.com

# Usage:
# Create an Annotations from PDF file
# Save as text file
# perl countann5 < input.txt [ > output.txt ] 
#
use Text::Wrap;     # import wrap()

$Text::Wrap::columns = 76;  # wrap output lines


# fill in this array with the starting page numbers of each chapter
# the first one is Chapter 0 used for preliminary sections
# chapter numbers use proper page numbers so chapter 1 should always be
# page 1 in a T10 standard (the Scope section)
@chapterstartpage = (0, 1, 3, 5, 19, 65, 86, 127, 213, 227, 266, 317);

sub report_chapter {
	my ($page) = @_;
	my $lastchapter = " Chapter 0";		# start with space so it sorts first
	my $chapter = 0;
	return $lastchapter if ($page =~ /[ivx]/);
	foreach $startpage (@chapterstartpage) {
		if ($page >= $startpage) {
			$lastchapter = " Chapter $chapter";
		}
		$chapter++;
	}
	return $lastchapter;
}

$init_chapter = 0;
foreach (@chapterstartpage) {
	$authors{" Chapter $init_chapter"} = 0;
	$init_chapter++;
}

while (<>) {

    # most lines start with Page
    # sometimes it gets concatenated to the previous line with .Page
    if (/^Page: (\S+)/) {
	if (($foundstat == 0) && ($author) && ($chapter) && ($page)) { 
		print "No stats for author=$author chapter=$chapter page=$page\n";
	}; 
        $page = $1;
	$chapter = report_chapter ($page);

    } elsif (/^Sequence number: (\d+)/) {
         $item = $1;

    } elsif (/^Type: /) {

    } elsif (/^Author: (.+)/) {
	if (($foundstat == 0) && ($author) && ($chapter) && ($page)) { 
		print "No stats for author=$author chapter=$chapter page=$page\n";
	}; 
	 $author = $1;
	 $author =~ s/ $//;		# remove pesky trailing space in some lines
	 $authors{$author}++;		# count per author
	 $authors{$chapter}++;		# count per author entry (page entries are shared)
         $annotation_count++;
	 $foundstat = 0;

    } elsif (/^Date:/) {

    } else {
	if (/ACCEPT - DONE/) {
		print "Extra acceptdone: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$acceptdone++;
		$acceptdone{$author}++;
		$acceptdone{$chapter}++;
		$foundstat = 1;
	} elsif ((/ACCEPT - TODO/) && (/last edit/)) {
		print "Extra acceptlast: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$acceptlast++;
		$acceptlast{$author}++;
		$acceptlast{$chapter}++;
		$foundstat = 1;
	} elsif (/ACCEPT - TODO/) {
		print "Extra accepttodo: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$accepttodo++;
		$accepttodo{$author}++;
		$accepttodo{$chapter}++;
		$foundstat = 1;
	} elsif (/^ACCEPT/) {
		# print "$author  $chapter  $_";
		print "Extra accept: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		print "acceptonly: author=$author chapter=$chapter page=$page\n";
		$acceptonly++;
		$acceptonly{$author}++;
		$acceptonly{$chapter}++;
		$foundstat = 1;
	} elsif (/REJECT - TODO/) {
		print "Extra rejecttodo: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$rejecttodo++;
		$rejecttodo{$author}++;
		$rejecttodo{$chapter}++;
		$foundstat = 1;
	} elsif (/^REJECT/) {
		print "Extra reject: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$rejectonly++;
		$rejectonly{$author}++;
		$rejectonly{$chapter}++;
		$foundstat = 1;
	} elsif (/^REFER PORT WG - TODO/) {
		print "Extra porttodo: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$porttodo++;
		$porttodo{$author}++;
		$porttodo{$chapter}++;
		$foundstat = 1;
	} elsif ((/^REFER/) && (!$foundstat)) {
		print "Extra refer: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$refer++;
		$refer{$author}++;
		$refer{$chapter}++;
		$foundstat = 1;
	} elsif (/TODO/) {
		print "Extra todo: author=$author chapter=$chapter page=$page\n" if ($foundstat == 1);
		$todoonly++;
		$todoonly{$author}++;
		$todoonly{$chapter}++;
		$foundstat = 1;
        }
    }
}

# Print per-author statistics
# This includes per-chapter statistics since chapter headers are also
# stored in the %authors hash
print "Per-author statistics\n";
foreach $author (sort keys %authors) {
	print "$author\n";
	print "  Total comments: " . $authors{$author} . "\n";

	print "   Accept - Done: $acceptdone{$author}\n" if $acceptdone{$author};
	print "   Accept - Todo: $accepttodo{$author}\n" if $accepttodo{$author};
	print "   Accept - Last: $acceptlast{$author}\n" if $acceptlast{$author};
	print "   Accept (only): $acceptonly{$author}\n" if $acceptonly{$author};
	print "   Reject - Todo: $rejecttodo{$author}\n" if $rejecttodo{$author};
	print "   Reject (only): $rejectonly{$author}\n" if $rejectonly{$author};
	print "       Port Todo: $porttodo{$author}\n" if $porttodo{$author};
	print "           Refer: $refer{$author}\n" if $refer{$author};
	print "     Todo (only): $todoonly{$author}\n" if $todoonly{$author};
	$totalworked = $acceptdone{$author} + $acceptlast{$author} + $accepttodo{$author} + $acceptonly{$author} + $rejecttodo{$author} + $rejectonly{$author} + $porttodo{$author} + $refer{$author} + $todoonly{$author};
	print "    Total worked: $totalworked\n";
	print "            Left: " . ($authors{$author} - $totalworked) . "\n";
}


# Print global statistics
print "\n----------------\n";

print "Total comments: $annotation_count\n";
print " Accept - Done: $acceptdone\n";
print " Accept - Last: $acceptlast\n";
print " Accept - Todo: $accepttodo\n";
print " Accept (only): $acceptonly\n";
print " Reject - Todo: $rejecttodo\n";
print " Reject (only): $rejectonly\n";
print "     Port todo: $porttodo\n";
print "         Refer: $refer\n";
print "   Todo (only): $todoonly\n";
$totalworked = $acceptdone + $acceptlast + $accepttodo + $acceptonly + $rejecttodo + $rejectonly + $porttodo + $refer + $todoonly;
print "          Left: " . ($annotation_count - $totalworked) . "\n";
printf ("  Total worked: %d (%2.2f\%)\n", $totalworked, ($totalworked / $annotation_count) * 100);

